@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /home/t-ishii/repository/pgpool/main.c,v 1.11 2004/05/06 06:30:35 t-ishii Exp $
+ * $Header: /home/t-ishii/repository/pgpool/main.c,v 1.14 2004/06/15 12:57:55 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -120,6 +120,7 @@ int main(int argc, char **argv)
 	if (pool_get_config(conf_file))
 	{
 		pool_error("Unable to get configuration. Exiting...");
+		mypid = getpid();
 		myexit(1);
 	}
 
@@ -207,7 +208,8 @@ int main(int argc, char **argv)
 
 static void usage(void)
 {
-	fprintf(stderr, "pgpool version %s, a generic connection pool/replication server for PostgreSQL\n\n", PGPOOLVERSION);
+	fprintf(stderr, "pgpool version %s,\n",	PGPOOLVERSION);
+	fprintf(stderr, "  a generic connection pool/replication/load balance server for PostgreSQL\n\n");
 	fprintf(stderr, "usage: pgpool [-f config_file][-n][-d][-h][stop]\n");
 	fprintf(stderr, "  config_file default path: %s/%s\n",DEFAULT_CONFIGDIR, POOL_CONF_FILE_NAME);
 	fprintf(stderr, "  -n: don't run in daemon mode. does not detatch control tty\n");
@@ -474,6 +476,7 @@ void notice_backend_error(void)
 {
 	pid_t parent = getppid();
 
+	pool_log("notice_backend_error: fail over request from pid %d", getpid());
 	kill(parent, SIGUSR1);
 	/* avoid race conditon with SIGCHLD */
 	sleep(1);
