@@ -1,7 +1,7 @@
 /* -*-pgsql-c-*- */
 /*
  *
- * $Header: /home/t-ishii/repository/pgpool/pool.h,v 1.19 2004/06/15 12:57:55 t-ishii Exp $
+ * $Header: /home/t-ishii/repository/pgpool/pool.h,v 1.20 2004/07/16 10:59:57 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -202,7 +202,9 @@ typedef struct {
     int pid;	/* backend pid */
     int key;	/* cancel key */
     POOL_CONNECTION	*con;
-	time_t closetime;	/* absolute time in seconds when the connection closed */
+	time_t closetime;	/* absolute time in second when the connection closed
+						 * if 0, that means the connection is under use.
+						 */
 } POOL_CONNECTION_POOL_SLOT;
 
 #define MAX_CONNECTION_SLOTS 2
