@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /home/t-ishii/repository/pgpool/pool_connection_pool.c,v 1.21 2004/06/30 08:01:52 t-ishii Exp $
+ * $Header: /home/t-ishii/repository/pgpool/pool_connection_pool.c,v 1.22 2004/07/16 11:03:26 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -43,6 +43,7 @@
 POOL_CONNECTION_POOL *pool_connection_pool;	/* connection pool */
 
 static POOL_CONNECTION_POOL_SLOT *create_cp(POOL_CONNECTION_POOL_SLOT *cp, int secondary_backend);
+static POOL_CONNECTION_POOL *new_connection(POOL_CONNECTION_POOL *p);
 
 /*
 * initialize connection pools. this should be called once at the startup.
@@ -129,6 +130,8 @@ POOL_CONNECTION_POOL *pool_create_cp(void)
 {
 	int i;
 	int fd;
+	time_t closetime;
+	POOL_CONNECTION_POOL *oldestp;
 
 	POOL_CONNECTION_POOL *p = pool_connection_pool;
 
@@ -141,48 +144,55 @@ POOL_CONNECTION_POOL *pool_create_cp(void)
 	for (i=0;i<pool_config.max_pool;i++)
 	{
 		if (MASTER_CONNECTION(p) == NULL)
+			return new_connection(p);
+		p++;
+	}
+
+	pool_debug("no empty connection slot was found");
+
+	/*
+	 * no empty connection slot was found. look for the oldest connection and discard it.
+	 */
+	oldestp = p = pool_connection_pool;
+	closetime = MASTER_CONNECTION(p)->closetime;
+	for (i=0;i<pool_config.max_pool;i++)
+	{
+		pool_debug("user: %s database: %s closetime: %d",
+				   MASTER_CONNECTION(p)->sp->user,
+				   MASTER_CONNECTION(p)->sp->database,
+				   MASTER_CONNECTION(p)->closetime);
+		if (MASTER_CONNECTION(p)->closetime < closetime)
 		{
-			/* create master connection */
-			MASTER_CONNECTION(p) = malloc(sizeof(POOL_CONNECTION_POOL_SLOT));
-			if (MASTER_CONNECTION(p) == NULL)
-			{
-				pool_error("pool_create_cp: malloc() failed");
-				return NULL;
-			}
-			create_cp(MASTER_CONNECTION(p), 0);
-
-			/* initialize Paramter Status save structure */
-			if (pool_init_params(&MASTER(p)->params))
-			{
-				return NULL;
-			}
-			p->num = 1;	/* number of slots */
-
-			/* create secondary connection */
-			if (pool_config.replication_enabled)
-			{
-				SECONDARY_CONNECTION(p) = malloc(sizeof(POOL_CONNECTION_POOL_SLOT));
-				if (SECONDARY_CONNECTION(p) == NULL)
-				{
-					pool_error("pool_create_cp: malloc() failed");
-					return NULL;
-				}
-				create_cp(SECONDARY_CONNECTION(p), 1);
-
-				/* initialize Paramter Status save structure */
-				if (pool_init_params(&MASTER(p)->params))
-				{
-					return NULL;
-				}
-
-				p->num++;	/* number of slots */
-			}
-
-			return p;
+			closetime = MASTER_CONNECTION(p)->closetime;
+			oldestp = p;
 		}
 		p++;
 	}
-	return NULL;
+
+	p = oldestp;
+	pool_send_frontend_exits(p);
+
+	pool_debug("discarding old %d th connection. user: %s database: %s", 
+			   oldestp - pool_connection_pool,
+			   MASTER_CONNECTION(p)->sp->user,
+			   MASTER_CONNECTION(p)->sp->database);
+
+	free(MASTER_CONNECTION(p)->sp->user);
+	free(MASTER_CONNECTION(p)->sp->database);
+	free(MASTER_CONNECTION(p)->sp->startup_packet);
+	pool_close(MASTER_CONNECTION(p)->con);
+	memset(p, 0, sizeof(POOL_CONNECTION_POOL));
+
+	if (pool_config.replication_enabled)
+	{
+		free(SECONDARY_CONNECTION(p)->sp->user);
+		free(SECONDARY_CONNECTION(p)->sp->database);
+		free(SECONDARY_CONNECTION(p)->sp->startup_packet);
+		pool_close(SECONDARY_CONNECTION(p)->con);
+		memset(p, 0, sizeof(POOL_CONNECTION_POOL));
+	}
+
+	return new_connection(p);
 }
 
 /*
@@ -409,3 +419,44 @@ static POOL_CONNECTION_POOL_SLOT *create_cp(POOL_CONNECTION_POOL_SLOT *cp, int s
 	cp->closetime = 0;
 	return cp;
 }
+
+static POOL_CONNECTION_POOL *new_connection(POOL_CONNECTION_POOL *p)
+{
+	/* create master connection */
+	MASTER_CONNECTION(p) = malloc(sizeof(POOL_CONNECTION_POOL_SLOT));
+	if (MASTER_CONNECTION(p) == NULL)
+	{
+		pool_error("pool_create_cp: malloc() failed");
+		return NULL;
+	}
+	create_cp(MASTER_CONNECTION(p), 0);
+
+			/* initialize Paramter Status save structure */
+	if (pool_init_params(&MASTER(p)->params))
+	{
+		return NULL;
+	}
+	p->num = 1;	/* number of slots */
+
+	/* create secondary connection */
+	if (pool_config.replication_enabled)
+	{
+		SECONDARY_CONNECTION(p) = malloc(sizeof(POOL_CONNECTION_POOL_SLOT));
+		if (SECONDARY_CONNECTION(p) == NULL)
+		{
+			pool_error("pool_create_cp: malloc() failed");
+			return NULL;
+		}
+		create_cp(SECONDARY_CONNECTION(p), 1);
+
+		/* initialize Paramter Status save structure */
+		if (pool_init_params(&MASTER(p)->params))
+		{
+			return NULL;
+		}
+
+		p->num++;	/* number of slots */
+	}
+
+	return p;
+}
