@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /home/t-ishii/repository/pgpool/pool_process_query.c,v 1.24 2004/07/04 03:12:42 t-ishii Exp $
+ * $Header: /home/t-ishii/repository/pgpool/pool_process_query.c,v 1.25 2004/07/16 11:02:35 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -679,6 +679,7 @@ static POOL_STATUS AsciiRow(POOL_CONNECTION *frontend,
 							short num_fields)
 {
 	static char nullmap[8192];
+	static char nullmap1[8192];
 	int nbytes;
 	int i;
 	unsigned char mask;
@@ -696,12 +697,22 @@ static POOL_STATUS AsciiRow(POOL_CONNECTION *frontend,
 	/* NULL map */
 	pool_read(MASTER(backend), nullmap, nbytes);
 	if (pool_write(frontend, nullmap, nbytes) < 0)
-	{
 		return POOL_END;
-	}
+
 	if (REPLICATION)
 	{
-		pool_read(SECONDARY(backend), nullmap, nbytes);
+		if (pool_read(SECONDARY(backend), nullmap1, nbytes) < 0)
+			return POOL_END;
+	}
+
+	if (memcmp(nullmap, nullmap1, nbytes))
+	{
+		/* XXX: NULLMAP maybe different among
+		   backends. If we were a paranoid, we have to treat
+		   this as a fatal error. However in the real world
+		   we'd better to adapt this situation. Just throw a
+		   log... */
+		pool_log("AsciiRow: NULLMAP differ between master and secondary");
 	}
 
 	mask = 0;
@@ -717,24 +728,28 @@ static POOL_STATUS AsciiRow(POOL_CONNECTION *frontend,
 			/* field size */
 			if (pool_read(MASTER(backend), &size, sizeof(int)) < 0)
 				return POOL_END;
-			if (REPLICATION)
-			{
-				/* XXX: field size maybe different between
-				   backends. If we were a paranoid, we have to treat
-				   this as a fatal error. However in the real world
-				   we'd better to adapt this situation. Just throw a
-				   log... */
+		}
 
-				if (pool_read(SECONDARY(backend), &size1, sizeof(int)) < 0)
-					return POOL_END;
+		if (REPLICATION && (mask & nullmap1[i/8]))
+		{
+			/* XXX: field size maybe different among
+			   backends. If we were a paranoid, we have to treat
+			   this as a fatal error. However in the real world
+			   we'd better to adapt this situation. Just throw a
+			   log... */
 
-				if (size != size1)
-					pool_log("AsciiRow: %d th field size does not match between master(%d) and secondary(%d)",
-							 i, ntohl(size), ntohl(size1));
-				size1 = ntohl(size1) - 4;
+			if (pool_read(SECONDARY(backend), &size1, sizeof(int)) < 0)
+				return POOL_END;
 
-			}
+			if (size != size1)
+				pool_log("AsciiRow: %d th field size does not match between master(%d) and secondary(%d)",
+						 i, ntohl(size), ntohl(size1));
+			size1 = ntohl(size1) - 4;
+		}
 
+		if (mask & nullmap[i/8])
+		{
+			/* forward to frontend */
 			pool_write(frontend, &size, sizeof(int));
 			size = ntohl(size) - 4;
 
@@ -753,31 +768,33 @@ static POOL_STATUS AsciiRow(POOL_CONNECTION *frontend,
 					free(buf);
 					return POOL_END;
 				}
-				if (REPLICATION && size1 > 0)
-				{
-					buf1 = malloc(size1);
-					if (buf1 == NULL)
-					{
-						pool_error("AsciiRow: out of memory");
-						return POOL_END;
-					}
-					if (pool_read(SECONDARY(backend), buf1, size1) < 0)
-					{
-						free(buf1);
-						return POOL_END;
-					}
-					free(buf1);
-				}
-				pool_write(frontend, buf, size);
-				snprintf(msgbuf, Min(sizeof(msgbuf), size), "%s", buf);
-				free(buf);
 			}
-			else
-			{
-				*msgbuf = '\0';
-			}
-			pool_debug("AsciiRow: len:%d data: %s", size, msgbuf);
 		}
+
+		if (REPLICATION && size1 > 0 && (mask & nullmap1[i/8]))
+		{
+			buf1 = malloc(size1);
+			if (buf1 == NULL)
+			{
+				pool_error("AsciiRow: out of memory");
+				return POOL_END;
+			}
+			if (pool_read(SECONDARY(backend), buf1, size1) < 0)
+			{
+				free(buf1);
+				return POOL_END;
+			}
+			free(buf1);
+		}
+
+		if ((mask & nullmap[i/8]) &&  size > 0)
+		{
+			pool_write(frontend, buf, size);
+			snprintf(msgbuf, Min(sizeof(msgbuf), size+1), "%s", buf);
+			free(buf);
+			pool_debug("AsciiRow: len: %d data: %s", size, msgbuf);
+		}
+
 		mask >>= 1;
 	}
 
