@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
-* $Header: /home/t-ishii/repository/pgpool/pool_stream.c,v 1.18 2004/05/11 09:27:14 t-ishii Exp $
+* $Header: /home/t-ishii/repository/pgpool/pool_stream.c,v 1.20 2004/06/08 10:39:20 t-ishii Exp $
 *
 * pgpool: a language independent connection pool server for PostgreSQL 
 * written by Tatsuo Ishii
@@ -56,23 +56,10 @@ POOL_CONNECTION *pool_open(int fd)
 
 	memset(cp, 0, sizeof(*cp));
 
-#ifdef NOT_USED
-	cp->read_fd = fdopen(fd, "r");
-	if (cp->read_fd == NULL)
-	{
-		pool_error("pool_open: fdopen failed: %s",strerror(errno));
-		free(cp);
-		return NULL;
-	}
-#endif
-
 	cp->write_fd = fdopen(fd, "w");
 	if (cp->write_fd == NULL)
 	{
 		pool_error("pool_open: fdopen failed: %s",strerror(errno));
-#ifdef NOT_USED
-		fclose(cp->read_fd);
-#endif
 		free(cp);
 		return NULL;
 	}
@@ -89,6 +76,8 @@ POOL_CONNECTION *pool_open(int fd)
 	cp->len = 0;
 	cp->sbuf = NULL;
 	cp->sbufsz = 0;
+	cp->buf2 = NULL;
+	cp->sbufsz = 0;
 
 	cp->fd = fd;
 	return cp;
@@ -100,12 +89,12 @@ POOL_CONNECTION *pool_open(int fd)
 void pool_close(POOL_CONNECTION *cp)
 {
 	fclose(cp->write_fd);
-#ifdef NOT_USED
-	fclose(cp->read_fd);
-	close(cp->fd);
-#endif
 	free(cp->hp);
-	free(cp->sbuf);
+	if (cp->sbuf)
+		free(cp->sbuf);
+	if (cp->buf2)
+		free(cp->buf2);
+	pool_discard_params(&cp->params);
 	free(cp);
 }
 
@@ -184,23 +173,102 @@ int pool_read(POOL_CONNECTION *cp, void *buf, int len)
 		len -= readlen;
 	}
 
-#ifdef DEBUG
-	if (readlen == 1)
-	{
-		pool_debug("pool read %d byte (%c)", len, *((char *)buf));
-	}
-#endif
-
 	return 0;
 }
 
+/*
+* read exactly len bytes from cp
+* returns buffer address on success otherwise NULL.
+*/
+char *pool_read2(POOL_CONNECTION *cp, int len)
+{
+	char *buf;
+	int req_size;
+	int alloc_size;
+	int consume_size;
+	int readlen;
+
+	req_size = cp->len + len;
+
+	if (req_size > cp->bufsz2)
+	{
+		alloc_size = ((req_size+1)/READBUFSZ+1)*READBUFSZ;
+		cp->buf2 = realloc(cp->buf2, alloc_size);
+		if (cp->buf2 == NULL)
+		{
+			pool_error("pool_read2: failed to realloc");
+			exit(1);
+		}
+		cp->bufsz2 = alloc_size;
+	}
+
+	buf = cp->buf2;
+
+	consume_size = consume_pending_data(cp, buf, len);
+	len -= consume_size;
+	buf += consume_size;
+
+	while (len > 0)
+	{
+		if (cp->issecondary_backend)
+		{
+			if (pool_check_fd(cp, 0))
+			{
+				pool_log("pool_read2: secondary data is not ready. abort this session");
+				exit(1);
+			}
+		}
+
+		readlen = read(cp->fd, buf, len);
+		if (readlen == -1)
+		{
+			pool_error("pool_read2: read failed (%s)", strerror(errno));
+
+			if (cp->isbackend)
+			{
+			    /* fatal error, notice to parent and exit */
+			    notice_backend_error();
+			    exit(1);
+			}
+			else
+			{
+			    return NULL;
+			}
+		}
+		else if (readlen == 0)
+		{
+			pool_error("pool_read2: EOF encountered");
+
+			if (cp->isbackend)
+			{
+			    /* fatal error, notice to parent and exit */
+			    notice_backend_error();
+				exit(1);
+			}
+			else
+			{
+				/*
+				 * if backend offers authentication method, frontend could close connection
+				 */
+				return NULL;
+			}
+		}
+
+		buf += readlen;
+		len -= readlen;
+	}
+
+	return cp->buf2;
+}
+
 /*
 * write len bytes from cp
 * returns 0 on success otherwise -1.
 */
 int pool_write(POOL_CONNECTION *cp, void *buf, int len)
 {
-	fwrite(buf, len, 1, cp->write_fd);
+	if (!cp->no_forward)
+		fwrite(buf, len, 1, cp->write_fd);
 
 	return 0;
 }
